#!/usr/bin/python

import sys, getopt, re, os, fnmatch, shlex

################################################
# Vypise napovedu a skonci 
#
################################################
def printHelp():
    print 'Skript pro generovani zapouzdrene entity.'
    print 'wrapper.py -f <entity_file> -o <output_file> -n <wrapper_name> -d <directory_to_search> -w'
    print '-f <entity_file>           soubor obsahujici entitu, pro kterou se udela wrapper'
    print '-o <output_file>           soubor obsahujici wrapper entity'
    print '-n <wrapper_name>          nazev zapouzdrene entity'
    print '-d <directory_to_search>   adresare, kde se budou vyhledava nezname typy signalu'
    print '-r <route_entity_file>     pokud je zadany, vytvori wrapper route komponenty, pri tom' 
    print '                           nacte zadanou entitu a podle ni vytvori rozhrani route komponenty'
    print '-w                         vytvoreni souboru width.tmp se sirkami INPUT a OUTPUT signalu'
 
################################################
# Vytvori hlavicku wrapperu
#
################################################
def getHeader(entityName):
    fileString=''
    fileString=fileString+'-- Toto je automaticky generovany VHDL popis zapouzdrene jednotky '+entityName+'.\n'
    fileString=fileString+'\n'
    return fileString

################################################
# Vytvori hlavicku wrapperu
#
################################################
def getRouteHeader(entityName, routeEntityName):
    fileString=''
    fileString=fileString+'-- Toto je automaticky generovany VHDL popis zapouzdrene jednotky '+routeEntityName+' s routovanim\n'
    fileString=fileString+'-- s vnejsim rozhrani podle jednotky '+entityName+'.\n'
    fileString=fileString+'\n'
    return fileString

################################################
# Hlavni funkce
#
################################################
def main(argv):
    entityFileName = ''
    wrapperName = ''
    outputFileName=wrapperName+'.vhd'
    directoriesToSearch=()
    routeEntityFileName=''
    createWidthFile=False
    try:
        opts, args = getopt.getopt(argv,"hf:o:n:d:r:w",["entity_file=","output_file=","wrapper_name=","directories_to_search=","route_entity_file=","create_width_file"])
    except getopt.GetoptError:
        printHelp()
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            printHelp()
            sys.exit()
        elif opt in ("-f", "--entity_file"):
            if os.path.isfile(arg): 
                entityFileName = arg
            else:
                print 'Nebyl zadan existujici soubor s entitou. (',arg,')'
                printHelp()
                sys.exit(2)
        elif opt in ("-n", "--wrapper_name"):
            wrapperName = arg
            #outputFileName = wrapperName+'.vhd'
        elif opt in ("-o", "--output_file"):
            outputFileName = arg
        elif opt in ("-d", "--directories_to_search"):
            directoriesToSearch = arg.split()
        elif opt in ("-r", "--route_entity_file"):
            routeEntityFileName = arg        
        elif opt in ("-w", "--create_width_file"):
            createWidthFile = True

    if not os.path.isfile(entityFileName): 
        print 'Nebyl zadan existujici soubor s entitou. (',entityFileName,')'
        printHelp()
        sys.exit(2)

    for directory in directoriesToSearch:
        if not os.path.isdir(directory): 
            print 'Zadan neexistujici adresar pro hledani knihoven.'
            printHelp()
            sys.exit(2)

    entityFile = open(entityFileName, 'r')
    fileString = entityFile.read() 
    entityName, libraries, generics, ports = parseEntity(fileString, directoriesToSearch)
    entityFile.close()

    inputWidth=str(countWidth("in", ports))
    outputWidth=str(countWidth("out", ports))

    if(routeEntityFileName == ""):
        if(wrapperName == ''):
            wrapperName = entityName+"_wrapper"
        wrapperFile = open(outputFileName,'w')
        fileString = createEntityWrapper(wrapperName, entityName, libraries, generics, ports)
        #print "\n",outputFileName,"\n"
        #print fileString
        wrapperFile.write(fileString)
        wrapperFile.close()

        if(createWidthFile):
            widthFile = open("width.tmp",'w')
            fileString = "INPUT:"+inputWidth+"\n"
            fileString = fileString + "OUTPUT:"+outputWidth+"\n"
            widthFile.write(fileString)
            widthFile.close()
    else:
        routeEntityFile = open(routeEntityFileName, 'r')
        fileString=routeEntityFile.read() 
        fileString=fileString.replace("GENERIC_INPUT_WIDTH_PARAM", inputWidth)
        fileString=fileString.replace("GENERIC_OUTPUT_WIDTH_PARAM", outputWidth)
        routeEntityName, routeLibraries, routeGenerics, routePorts = parseEntity(fileString, directoriesToSearch)
        routeEntityFile.close()
        #print "Route ports"
        #print "============"
        #print routePorts
        #print ""
        if(wrapperName == ''):
            wrapperName = routeEntityName+"_wrapper"
        wrapperFile = open(outputFileName,'w')
        fileString = createRouteEntityWrapper(wrapperName, entityName, libraries, generics, ports, routeEntityName, routeLibraries, routeGenerics, routePorts)
        #print "\n",outputFileName,"\n"
        #print fileString
        wrapperFile.write(fileString)
        wrapperFile.close()

    #print "Entity ports"
    #print "============"
    #print ports
    #print ""

   


################################################
# Rozparsovani zadane entity 
#
# Parametry:
#   fileString           - VHD string ze souboru obsahujici entitu
#   directoriesToSearch  - pole obsahujici nazvy adresaru, kde se maji hledat nezname typy signalu
#
# Vracene hodnoty:
#   entityName   - nazev entity
#   libraries    - retezec s pouziti 
#   generics     - pole generickych hodnot, z nichz kazda hodnota generic obsahuje:
#     generic[0] - nazev
#     generic[1] - typ
#     generic[2] - defaultni hodnota
#   ports        - pole portu, kde kazda hodnota port obsahuje:
#     port[0]    - nazev
#     port[1]    - smer
#     port[2]    - typ
#     port[3]    - sirka
#     port[4]    - defaultni hodnota
#     port[5]    - zda je to puvodni definovany typ z entity (True pokud se hodnota vyskytovala v puvodni entite) 
#     port[6]    - zda se pocita jeho sirka (True jen u std_logic a std_logic_vector)              
################################################
def parseEntity(fileString, directoriesToSearch): 
    # odstraneni komentaru
    fileString = re.sub('\-\-.*', '', fileString)

    entityNames = re.findall(r"entity\s+(\S+)\s+", fileString, re.DOTALL | re.IGNORECASE)
    if(len(entityNames) != 1):
        print 'Problem s nacitanim entity.'
        sys.exit(2)
    entityName = entityNames[0]

    libraryGroups = re.findall(r"^(.*)\s+entity", fileString, re.DOTALL | re.IGNORECASE)
    if(len(libraryGroups) != 1):
        print 'Problem s nacitanim knihoven entity.'
        sys.exit(2)
    libraries = libraryGroups[0]
    #print "Knihovny:"
    #print libraries
    #print "konec"
    entities = re.findall(r"(entity\s+"+entityName+".*end\s+"+entityName+"\s*;)", fileString, re.DOTALL | re.IGNORECASE)
    
    if(len(entities) != 1):
        print 'Problem s nacitanim entity.'
        sys.exit(2)

    entityString = entities[0]

    # odstraneni komentaru
    #entityString = re.sub('\-\-.*', '', entityString)
    # naformatovani radku
    entityString = re.sub('(\t|[ ]|)*(\r|\n)+(\t|[ ])*', '\n', entityString)
    entityString = re.sub('\n+', '\n', entityString)
    
    #print 'Entity:'
    #print entityString

    genericString = re.findall(r"generic\s*\((.*?)\)\s*;\s*port", entityString, re.DOTALL | re.IGNORECASE)
    portString = re.findall(r"port\s*\((.*?)\)\s*;\s*end", entityString, re.DOTALL | re.IGNORECASE)

    generics = ()
    if(len(genericString) < 1):
        print 'Entita neobsahuje genericke parametry.'
    else:
        #print 'Generic:'
        #print genericString[0]
        generics = re.findall(r"([\S]+?)\s*:\s*(.*?)\s*(?:;|$|:=\s*([\S]+?)\s*(?:;|$))", genericString[0], re.DOTALL | re.IGNORECASE)
    #print generics

    ports = ()
    if(len(portString) < 1):
        print 'Entita neobsahuje definici portu!'
        sys.exit(2)
    
    #print 'Port:'
    #print portString[0]
    unprocessedPorts = re.findall(r"([\S]+)\s*:\s*(in|out|inout)\s+(.*?)\s*(?:;|$|:=\s*([\S]+?)\s*(?:;|$))", portString[0], re.DOTALL | re.IGNORECASE)
    ports=[]
    for unprocessedPort in unprocessedPorts:
        if(unprocessedPort[2][:9] != "std_logic"):
            #print "Neco jine nez std_logic: "+port[2]
            typeDefinition = ""
            for directoryToSearch in directoriesToSearch:
                typeDefinition=searchDirectoryForType(unprocessedPort[2], directoryToSearch)
                if(typeDefinition != ""):
                    break
            if(typeDefinition == ""):
                print 'Nelze nalezt typ '+unprocessedPort[2]+' v zadanych adresarich k hledani!'
                sys.exit(2)
            subPorts = re.findall(r"([\S]+)\s*:\s*(.*?)\s*(?:;|$|:=\s*([\S]+?)\s*(?:;|$))", typeDefinition, re.DOTALL | re.IGNORECASE)
            widthSum=0
            for subPort in subPorts:
                subPortList=list(subPort)
                subPortList[0]=unprocessedPort[0]+"."+subPortList[0]
                subPortList.insert(1,unprocessedPort[1])
                subPortList.append(False)
                subPortList.append(True)
                subPort=tuple(subPortList)
                #print "subPort:",subPort
                port, width=addWidth(subPort, generics)
                ports.append(port)
                widthSum = widthSum+width
            port=unprocessedPort
            portList = list(port)
            portList.insert(3,widthSum)
            portList.append(True)
            portList.append(False)
            port=tuple(portList)
            ports.append(port)
        else:
            port, width=addWidth(unprocessedPort, generics)
            portList = list(port)
            portList.append(True)
            portList.append(True)
            port = tuple(portList)
            ports.append(port)
    #print ports

    return entityName, libraries, generics, ports


################################################
# Vytvoreni VHD retezce entity wrapperu pod udaju ziskanych ze zadane entity 
#
# Parametry:
#   wrapperName  - nazev entity wrapperu
#   entityName   - nazev entity, pro kterou se vytvari wrapper
#   libraries    - retezec s knihovnami
#   generics     - pole generickych hodnot, z nichz kazda hodnota generic obsahuje:
#     generic[0] - nazev
#     generic[1] - typ
#     generic[2] - defaultni hodnota
#   ports        - pole portu, kde kazda hodnota port obsahuje:
#     port[0]    - nazev
#     port[1]    - smer
#     port[2]    - typ
#     port[3]    - sirka
#     port[4]    - defaultni hodnota
#     port[5]    - zda je to puvodni definovany typ z entity (True pokud se hodnota vyskytovala v puvodni entite) 
#     port[6]    - zda se pocita jeho sirka (True jen u std_logic a std_logic_vector)         
#
# Vracene hodnoty:
#   fileString           - VHD string ze souboru obsahujici entitu wrapperu    
################################################
def createEntityWrapper(wrapperName, entityName, libraries, generics, ports):
    inputPortsWidth = countWidth("in",ports)
    outputPortsWidth = countWidth("out",ports)
    
    # definice knihoven
    fileString =getHeader(entityName)
    fileString+=joinLibraries(libraries,"")+"\n"
    
    # definice entity
    fileString+="entity "+wrapperName+" is\n"
    fileString+="    generic (\n"

    for genericIndex in range(0,len(generics)):
        generic=generics[genericIndex]
        if(generic[2] != ''):
            fileString+="        "+generic[0]+"\t: "+generic[1]+" := "+generic[2]+""
        else:
            fileString+="        "+generic[0]+"\t: "+generic[1]+""
        if(genericIndex < len(generics)-1):
            fileString+=";\n"
        else:
            fileString+="\n"

    fileString+="    );\n"
    fileString+="    port (\n"
    fileString+="        input  : in  std_logic_vector("+str(inputPortsWidth-1)+" downto 0);\n"
    fileString+="        output : out std_logic_vector("+str(outputPortsWidth-1)+" downto 0)\n"
    fileString+="    );\n"
    fileString+="end "+wrapperName+";\n"
    fileString+="\n"
   
    #definice architektury
    fileString+="architecture structural of "+wrapperName+" is\n"
    fileString+="    component "+entityName+" is\n"
    fileString+="    generic (\n"
    for genericIndex in range(0,len(generics)):
        generic=generics[genericIndex]
        if(generic[2] != ''):
            fileString+="        "+generic[0]+"\t: "+generic[1]+" := "+generic[2]+""
        else:
            fileString+="        "+generic[0]+"\t: "+generic[1]+""
        if(genericIndex < len(generics)-1):
            fileString+=";\n"
        else:
            fileString+="\n"
    fileString+="    );\n"
    fileString+="    port (\n"
    for portIndex in range(0,len(ports)):
        port=ports[portIndex]
        if(port[5] == False):
            continue
        if(port[4] != ''):
            fileString+="        "+port[0]+"\t: "+port[1]+" "+port[2]+" := "+port[4]+""
        else:
            fileString+="        "+port[0]+"\t: "+port[1]+" "+port[2]+""
        if(portIndex < len(ports)-1):
            fileString+=";\n"
        else:
            fileString+="\n"
    fileString+="    );\n"
    fileString+="    end component;\n"
    fileString+="    \n"

    # deklarace signalu
    for portIndex in range(0,len(ports)):
        port=ports[portIndex]
        if(port[5] == False):
            continue
        fileString+="    signal "+port[0].lower() +"_sig\t: " + port[2] + ";\n"
    fileString+="    \n"

    # telo architektury
    fileString+="begin\n"
    fileString+="    \n"
    fileString+="    "+entityName+"_inst : "+entityName+"\n"
    fileString+="    generic map (\n"
    for genericIndex in range(0,len(generics)):
        generic=generics[genericIndex]
        fileString+="        "+generic[0]+"\t=> "+generic[0]+""
        if(genericIndex < len(generics)-1):
            fileString+=",\n"
        else:
            fileString+="\n"
    fileString+="    )\n"
    fileString+="    port map (\n"
    for portIndex in range(0,len(ports)):
        port=ports[portIndex]
        if(port[5] == False):
            continue
        fileString+="        "+port[0]+"\t=> "+port[0].lower() +"_sig"
        if(portIndex < len(ports)-1):
            fileString+=",\n"
        else:
            fileString+="\n"
    fileString+="    );\n"
    fileString+="    \n"

    # napojeni vstupu a vystupu
    pointer=inputPortsWidth
    for portIndex in range(0,len(ports)):
        port=ports[portIndex]
        if(port[1].lower() != "in"):
            continue
        if(port[6] != True):
            continue
        fileString+="    "+port[0].lower()+"_sig <= "
        if(port[3] == 1):
            fileString+="input("+str(pointer-1)+");\n"
        else:
            fileString+="input("+str(pointer-1)+" downto "+str(pointer-port[3])+");\n"
        
        pointer=pointer-port[3]

    fileString+="    \n"

    pointer=outputPortsWidth
    for portIndex in range(0,len(ports)):
        port=ports[portIndex]
        if(port[1].lower() != "out"):
            continue
        if(port[6] != True):
            continue
        if(port[3] == 1):
            fileString+="    output("+str(pointer-1)+")"
        else:
            fileString+="    output("+str(pointer-1)+" downto "+str(pointer-port[3])+")"
        fileString+=" <= "+port[0].lower()+"_sig;\n"
        pointer=pointer-port[3]

    fileString+="    \n"

    fileString+="end structural\n"
    return fileString

################################################
# Spoji retezce s knihovnami ze dvou souboru do jednoho retezce (udela jejich sjednoceni)
#
# Parametry:
#   libraryString1    - retezec s knihovnami prvniho souboru
#   libraryString2    - retezec s knihovnami druheho souboru
#
# Vracene hodnoty:
#   libraryString     - retezec se sloucenymi knihovnami
################################################
def joinLibraries(libraryString1, libraryString2):
    libraries=[]
    libraryString1 = re.sub('\r\n', '\n', libraryString1)
    libraryString1 = re.sub('\n\s*\n', '\n', libraryString1)
    libraryString1 = re.sub('^\s*\n', '', libraryString1)
    libraryString1 = re.sub('\n\s*$', '', libraryString1)
    libraries1=libraryString1.split('\n')
    libraryString2 = re.sub('\r\n', '\n', libraryString2)
    libraryString2 = re.sub('\n\s*\n', '\n', libraryString2)
    libraryString2 = re.sub('^\s*\n', '', libraryString2)
    libraryString2 = re.sub('\n\s*$', '', libraryString2)
    libraries2=libraryString2.split('\n')
    
    for library1 in libraries1:
        libraries.append(library1.lower())
    for library2 in libraries2:
        if library2.lower() in libraries:
            continue
        else:
            libraries.append(library2.lower())

    libraryString = "\n".join(libraries) + "\n"
    return libraryString



################################################
# Vytvoreni VHD retezce wrapperu route entity s rozhranim odpovidajici zadane entite 
#
# Parametry:
#   wrapperName       - nazev entity wrapperu
#   entityName        - nazev entity, podle nize se vytvari rozhrani route wrapperu
#   libraries         - retezec s knihovnami entity
#   generics          - pole generickych hodnot entity, z nichz kazda hodnota generic obsahuje:
#     generic[0]      - nazev
#     generic[1]      - typ
#     generic[2]      - defaultni hodnota
#   ports             - pole portu entity, kde kazda hodnota port obsahuje:
#     port[0]         - nazev
#     port[1]         - smer
#     port[2]         - typ
#     port[3]         - sirka
#     port[4]         - defaultni hodnota
#     port[5]         - zda je to puvodni definovany typ z entity (True pokud se hodnota vyskytovala v puvodni entite) 
#     port[6]         - zda se pocita jeho sirka (True jen u std_logic a std_logic_vector)    
#   routeEntityName   - nazev route entity, pro kterou se vytvari wrapper
#   routeLibraries    - retezec s knihovnami route entity
#   routeGenerics     - pole generickych hodnot route entity, tvorena stejnymi hodnotami jako generics vyse
#   routePorts        - pole portu route entity, tvorena stejnymi hodnotami jako ports vyse        
#
# Vracene hodnoty:
#   fileString        - VHD string ze souboru obsahujici wrapper route entity    
################################################
def createRouteEntityWrapper(wrapperName, entityName, libraries, generics, ports, routeEntityName, routeLibraries, routeGenerics, routePorts):
    inputPortsWidth = countWidth("in",ports)
    outputPortsWidth = countWidth("out",ports)
    
    # definice knihoven
    fileString =getRouteHeader(entityName,routeEntityName)
    fileString+=joinLibraries(libraries,routeLibraries)+"\n"

    # definice entity
    fileString+="entity "+wrapperName+" is\n"
    
    #fileString+="    generic (\n"
    #for routeGenericIndex in range(0,len(routeGenerics)):
    #    routeGeneric=routeGenerics[routeGenericIndex]
    #    if(routeGeneric[2] != ''):
    #        fileString+="        "+routeGeneric[0]+"\t: "+routeGeneric[1]+" := "+routeGeneric[2]+""
    #    else:
    #        fileString+="        "+routeGeneric[0]+"\t: "+routeGeneric[1]+""
    #    if(routeGenericIndex < len(routeGenerics)-1):
    #        fileString+=";\n"
    #    else:
    #        fileString+="\n"
    #fileString+="    );\n"

    fileString+="    port (\n"
    for portIndex in range(0,len(ports)):
        port=ports[portIndex]
        # pokud neni port kompletni typ (ale jen jeho cast napr: typ.signalA) 
        if(port[5] == False):
            continue
        # pokud je port inicializovany (inicializacni hodnota neni prazdna)
        if(port[4] != ''):
            fileString+="        "+port[0]+"\t: "+port[1]+" "+port[2]+" := "+port[4]+""
        else:
            fileString+="        "+port[0]+"\t: "+port[1]+" "+port[2]+""
        if(portIndex < len(ports)-1):
            fileString+=";\n"
        else:
            fileString+="\n"
    for routePortIndex in range(0,len(routePorts)):
        routePort=routePorts[routePortIndex]
        # pokud neni port kompletni typ (ale jen jeho cast napr: typ.signalA) 
        if(routePort[5] == False):
            continue
        # INPUT a OUTPUT nejdou na rozhrani wrapperu, ostatni ano
        if(routePort[0] == "INPUT" or routePort[0] == "OUTPUT"):
            continue
        # pokud je port inicializovany (inicializacni hodnota neni prazdna)
        if(routePort[4] != ''):
            fileString+="        "+routePort[0]+"\t: "+routePort[1]+" "+routePort[2]+" := "+routePort[4]+""
        else:
            fileString+="        "+routePort[0]+"\t: "+routePort[1]+" "+routePort[2]+""
        if(routePortIndex < len(routePorts)-1):
            fileString+=";\n"
        else:
            fileString+="\n"
    fileString+="    );\n"
    fileString+="end "+wrapperName+";\n"
    fileString+="\n"
   
    #definice architektury
    fileString+="architecture structural of "+wrapperName+" is\n"
    fileString+="    component "+routeEntityName+" is\n"
    fileString+="    generic (\n"
    foundInput=False
    foundOutput=False
    for routeGenericIndex in range(0,len(routeGenerics)):
        routeGeneric=routeGenerics[routeGenericIndex]
        if(routeGeneric[0] == "INPUT_WIDTH"):
            foundInput=True
        elif(routeGeneric[0] == "OUTPUT_WIDTH"):
            foundOutput=True

    if(foundInput == False or foundOutput == False):
        print 'Route entita neobsahuje genericke paramentry INPUT_WIDTH a OUTPUT_WIDTH!'
        sys.exit(2)

    fileString+="        INPUT_WIDTH\t:= integer;\n"
    fileString+="        OUTPUT_WIDTH\t:= integer\n"
    fileString+="    );\n"
    fileString+="    port (\n"
    for routePortIndex in range(0,len(routePorts)):
        routePort=routePorts[routePortIndex]
        if(routePort[5] == False):
            continue
        if(routePort[4] != ''):
            fileString+="        "+routePort[0]+"\t: "+routePort[1]+" "+routePort[2]+" := "+routePort[4]+""
        else:
            fileString+="        "+routePort[0]+"\t: "+routePort[1]+" "+routePort[2]+""
        if(routePortIndex < len(routePorts)-1):
            fileString+=";\n"
        else:
            fileString+="\n"
    fileString+="    );\n"
    fileString+="    end component;\n"
    fileString+="    \n"

    # deklarace signalu
    #for portIndex in range(0,len(ports)):
    #    port=ports[portIndex]
    #    if(port[5] == False):
    #        continue
    #    fileString+="    signal "+port[0].lower() +"_sig\t: " + port[2] + ";\n"
    #fileString+="    \n"

    for routePortIndex in range(0,len(routePorts)):
        routePort=routePorts[routePortIndex]
        if(routePort[5] == False):
            continue
        fileString+="    signal "+routePort[0].lower() +"_sig\t: " + routePort[2] + ";\n"
    fileString+="    \n"


    # telo architektury
    fileString+="begin\n"
    fileString+="    \n"
    fileString+="    "+routeEntityName+"_inst : "+routeEntityName+"\n"
    fileString+="    generic map (\n"
    fileString+="        INPUT_WIDTH\t=> "+str(inputPortsWidth)+",\n"
    fileString+="        OUTPUT_WIDTH\t=> "+str(outputPortsWidth)+"\n"
    fileString+="    )\n"
    fileString+="    port map (\n"
    for routePortIndex in range(0,len(routePorts)):
        routePort=routePorts[routePortIndex]
        if(routePort[5] == False):
            continue
        fileString+="        "+routePort[0]+"\t=> "+routePort[0].lower() +"_sig"
        if(routePortIndex < len(routePorts)-1):
            fileString+=",\n"
        else:
            fileString+="\n"
    fileString+="    );\n"
    fileString+="    \n"

    # napojeni vstupu entity
    fileString+="    -- vstupy podle entity\n"
    pointer=inputPortsWidth
    for portIndex in range(0,len(ports)):
        port=ports[portIndex]
        if(port[1].lower() != "in"):
            continue
        if(port[6] != True):
            continue
        if(port[3] == 1):
            fileString+="    input_sig("+str(pointer-1)+")"
        else:
            fileString+="    input_sig("+str(pointer-1)+" downto "+str(pointer-port[3])+")"
        fileString+=" <= "+port[0]+";\n"

        pointer=pointer-port[3]

    fileString+="    \n"

    # napojeni vystupu entity
    fileString+="    -- vystupy podle entity\n"
    pointer=outputPortsWidth
    for portIndex in range(0,len(ports)):
        port=ports[portIndex]
        if(port[1].lower() != "out"):
            continue
        if(port[6] != True):
            continue
        fileString+="    "+port[0].lower()+"_sig <= "
        if(port[3] == 1):
            fileString+="output_sig("+str(pointer-1)+");\n"
        else:
            fileString+="output_sig("+str(pointer-1)+" downto "+str(pointer-port[3])+");\n"
        
        pointer=pointer-port[3]
    fileString+="    \n"

    # napojeni vstupu od FT architektury
    fileString+="    -- vstupy do route z FT architektury\n"
    for routePortIndex in range(0,len(routePorts)):
        routePort=routePorts[routePortIndex]
        if(routePort[1].lower() != "in"):
            continue
        if(routePort[6] != True):
            continue
        # INPUT a OUTPUT nejdou na rozhrani wrapperu, ostatni ano
        if(routePort[0] == "INPUT"):
            continue
        fileString+="    "+routePort[0].lower()+"_sig  <= "+routePort[0]+";\n"
    fileString+="    \n"

    # napojeni vystupu do FT architektury
    fileString+="    -- vystupy z route do FT architektury\n"
    for routePortIndex in range(0,len(routePorts)):
        routePort=routePorts[routePortIndex]
        if(routePort[1].lower() != "out"):
            continue
        if(routePort[6] != True):
            continue
        # INPUT a OUTPUT nejdou na rozhrani wrapperu, ostatni ano
        if(routePort[0] == "OUTPUT"):
            continue
        fileString+="    "+routePort[0]+" <= "+routePort[0].lower()+"_sig;\n"
    fileString+="    \n"

    fileString+="end structural\n"
    return fileString



################################################
# Prohleda rekurzivne zadane adresare, pokusi se v souborech najit zadany typ a vrati jeho VHD retezec 
#
# Parametry:
#   typeName     - nazev hledaneho typu
#   entityName   - pole obsahujici adresare k hledani         
#
# Vracene hodnoty:
#   fileString   - VHD retezec s telem hledaneho typu    
################################################
def searchDirectoryForType(typeName, directoryToSearch):
    typeDefinition=""
    typeDefinitions=()
    #os.chdir(directoryToSearch)
    for root, dirnames, fileNames in os.walk(directoryToSearch):
        for vhdFileName in fnmatch.filter(fileNames, '*.vhd'):
    #for vhdFiles in glob.glob( "*.vhd" ):
            #print root,vhdFileName
            vhdFile=open( os.path.join(root, vhdFileName), 'r' )
            content = vhdFile.read()
            vhdFile.close()
            typeDefinitions = re.findall(r"(type\s+"+typeName+"\s+is\s+record.*end\s+record\s*;)", content, re.DOTALL | re.IGNORECASE)
            if(len(typeDefinitions) > 0):
                break
        if(len(typeDefinitions) > 0):
            typeDefinition=typeDefinitions[0]
            #print typeDefinition
            break
    return typeDefinition


################################################
# Vyhodnoti vyraz zadany retezcem a obsahujici genericke parametry,
# ty vyhodnoti pomoci jejich dohledani v poli generickych hodnot 
#
# Parametry:
#   statement    - vyraz
#   generics     - pole obsahujici genericke hodnoty         
#
# Vracene hodnoty:
#   num          - numericka hodnota vyrazu    
################################################
def evalStatement(statement, generics):
    num=0
    tokens = list(shlex.shlex(statement))
    #print tokens
    for tokenIndex in range(0,len(tokens)):
        for generic in generics:
            if(tokens[tokenIndex] == generic[0]):
                tokens[tokenIndex] = generic[2]
    #print tokens 
    numberStatement="".join(tokens)
    num=eval(numberStatement)
    #print num
    return num


################################################
# Vypocita sirku portu s pomoci pole generickych hodnot, 
# prida ji jako treti prvek pole, ktere port popisuje
#
# Parametry:
#   port         - port, pro ktery se sirka pocita, v tomto miste obsahuje tyto hodnoty:
#     port[0]    - nazev
#     port[1]    - smer
#     port[2]    - typ
#     port[3]    - defaultni hodnota
#   generics     - pole obsahujici genericke hodnoty         
#
# Vracene hodnoty:
#   port         - popis portu (pole hodnot portu), na konci funkce obsahuje tyto hodnoty:
#     port[0]    - nazev
#     port[1]    - smer
#     port[2]    - typ
#     port[3]    - sirka
#     port[4]    - defaultni hodnota
#   width        - spocitana sirka
################################################
def addWidth(port, generics):
    width=0
    if(re.search("\s*std_logic(\s+|$)", port[2], re.DOTALL | re.IGNORECASE) != None):
        portList=list(port)
        portList.insert(3,1)
        port=tuple(portList)
    elif(re.search("\s*std_logic_vector\(.*\)(?:\s+|$)", port[2], re.DOTALL | re.IGNORECASE) != None):
        nums = re.findall("\s*std_logic_vector\(\s*(\S+)\s*(?:to|downto)\s*(\S+)\s*\)(?:\s+|$)", port[2], re.DOTALL | re.IGNORECASE)
        
        if(len(nums) > 0):
            firstNum=evalStatement(nums[0][0], generics)
            secondNum=evalStatement(nums[0][1], generics)
            width=abs(firstNum-secondNum)+1
            #print "toWidth ",firstNum," fromWidth ",secondNum," width ",width
            portList=list(port)
            portList.insert(3,width)
            port=tuple(portList)
        else:
            print 'Nelze zjistit sirku signalu ',port[0],"!"
            sys.exit(2)
    else:
        print 'Nepodporovany typ signalu ',port[2],"!"
        sys.exit(2)
    return port, width


################################################
# Vypocita celkovou sirku vstupu nebo vystupu pomoci pole portu
#
# Parametry:
#   direction    - smer, melo by byt bud in nebo out (inout nepodporuje)
#   ports        - pole portu, kde kazda hodnota port obsahuje:
#     port[0]    - nazev
#     port[1]    - smer
#     port[2]    - typ
#     port[3]    - sirka
#     port[4]    - defaultni hodnota
#     port[5]    - zda je to puvodni definovany typ z entity (True pokud se hodnota vyskytovala v puvodni entite) 
#     port[6]    - zda se pocita jeho sirka (True jen u std_logic a std_logic_vector)           
#
# Vracene hodnoty:
#   widthSum     - spocitana celkova sirka
################################################
def countWidth(direction, ports):
    widthSum=0
    for port in ports:
        if(port[6] == False):
            continue
        if(port[1].lower() == direction.lower()):
            widthSum=widthSum+port[3]
            #if(direction.lower() == "out"):
            #    print "Name:",port[0]," width:",port[3]," widthSum:",widthSum
    return widthSum




# volani funkce main s parametrama
main(sys.argv[1:])
